pub struct Regex<S: TypeString>(/* private fields */);Available on crate features
alloc and regex only.Trait Implementations
Sourceimpl<S: TypeString> Default for Regex<S>
impl<S: TypeString> Default for Regex<S>
Sourceimpl<S: TypeString, T: AsRef<str>> Predicate<T> for Regex<S>
impl<S: TypeString, T: AsRef<str>> Predicate<T> for Regex<S>
Sourcefn error() -> ErrorMessage
fn error() -> ErrorMessage
An error message to display when the predicate doesn’t hold.
Sourceimpl<S: TypeString, T: AsRef<str>> StatefulPredicate<T> for Regex<S>
impl<S: TypeString, T: AsRef<str>> StatefulPredicate<T> for Regex<S>
Sourceunsafe fn optimize(value: &T)
unsafe fn optimize(value: &T)
Applies a potentially unsafe optimization to call sites that can take advantage of
information provided by the predicate. This function is unused by
refined unless
the optimized feature is enabled. Read moreSourcefn error(&self) -> ErrorMessage
fn error(&self) -> ErrorMessage
An error message to display when the predicate doesn’t hold.